@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -27,6 +27,7 @@ import traceback
 import types
 import warnings
 import zlib
+from collections import defaultdict
 from inspect import getfullargspec
 from io import StringIO
 from random import Random
@@ -55,21 +56,25 @@ from hypothesis.errors import (
     InvalidArgument,
     MultipleFailures,
     NoSuchExample,
+    StopTest,
     Unsatisfiable,
     UnsatisfiedAssumption,
 )
 from hypothesis.executors import new_style_executor
 from hypothesis.internal.compat import (
+    PYPY,
     bad_django_TestCase,
     get_type_hints,
     int_from_bytes,
     qualname,
 )
-from hypothesis.internal.conjecture.data import ConjectureData, StopTest
-from hypothesis.internal.conjecture.engine import ConjectureRunner, sort_key
+from hypothesis.internal.conjecture.data import ConjectureData, Status
+from hypothesis.internal.conjecture.engine import ConjectureRunner
+from hypothesis.internal.conjecture.shrinker import sort_key
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.escalation import (
     escalate_hypothesis_internal_error,
+    get_interesting_origin,
     get_trimmed_traceback,
 )
 from hypothesis.internal.healthcheck import fail_health_check
@@ -77,13 +82,13 @@ from hypothesis.internal.reflection import (
     arg_string,
     convert_positional_arguments,
     define_function_signature,
-    deprecated_posargs,
     function_digest,
     get_pretty_function_description,
     impersonate,
     is_mock,
     proxies,
 )
+from hypothesis.internal.scrutineer import Tracer, explanatory_lines
 from hypothesis.reporting import (
     current_verbosity,
     report,
@@ -156,7 +161,7 @@ def seed(seed: Hashable) -> Callable[[TestFunc], TestFunc]:
     return accept
 
 
-def reproduce_failure(version, blob):
+def reproduce_failure(version: str, blob: bytes) -> Callable[[TestFunc], TestFunc]:
     """Run the example that corresponds to this data blob in order to reproduce
     a failure.
 
@@ -192,7 +197,7 @@ def decode_failure(blob):
     try:
         buffer = base64.b64decode(blob)
     except Exception:
-        raise InvalidArgument("Invalid base64 encoded string: %r" % (blob,))
+        raise InvalidArgument(f"Invalid base64 encoded string: {blob!r}")
     prefix = buffer[:1]
     if prefix == b"\0":
         return buffer[1:]
@@ -200,10 +205,10 @@ def decode_failure(blob):
         try:
             return zlib.decompress(buffer[1:])
         except zlib.error:
-            raise InvalidArgument("Invalid zlib compression for blob %r" % (blob,))
+            raise InvalidArgument(f"Invalid zlib compression for blob {blob!r}")
     else:
         raise InvalidArgument(
-            "Could not decode blob %r: Invalid start byte %r" % (blob, prefix)
+            f"Could not decode blob {blob!r}: Invalid start byte {prefix!r}"
         )
 
 
@@ -218,7 +223,7 @@ class WithRunner(MappedSearchStrategy):
         return self.mapped_strategy.do_draw(data)
 
     def __repr__(self):
-        return "WithRunner(%r, runner=%r)" % (self.mapped_strategy, self.runner)
+        return f"WithRunner({self.mapped_strategy!r}, runner={self.runner!r})"
 
 
 def is_invalid_test(name, original_argspec, given_arguments, given_kwargs):
@@ -376,14 +381,16 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs):
                     continue
                 break
             finally:
-                assert fragments_reported[0].startswith("Falsifying example")
-                fragments_reported[0] = fragments_reported[0].replace(
-                    "Falsifying example", "Falsifying explicit example", 1
-                )
+                if fragments_reported:
+                    assert fragments_reported[0].startswith("Falsifying example")
+                    fragments_reported[0] = fragments_reported[0].replace(
+                        "Falsifying example", "Falsifying explicit example", 1
+                    )
 
-            verbose_report(fragments_reported[0].replace("Falsifying", "Trying", 1))
-            for f in fragments_reported[1:]:
-                verbose_report(f)
+            if fragments_reported:
+                verbose_report(fragments_reported[0].replace("Falsifying", "Trying", 1))
+                for f in fragments_reported[1:]:
+                    verbose_report(f)
 
 
 def get_random_for_wrapped_test(test, wrapped_test):
@@ -519,6 +526,8 @@ class StateForActualGivenExecution:
         self.files_to_propagate = set()
         self.failed_normally = False
 
+        self.explain_traces = defaultdict(set)
+
     def execute_once(
         self, data, print_example=False, is_final=False, expected_failure=None
     ):
@@ -587,31 +596,12 @@ class StateForActualGivenExecution:
                                     for v in args:
                                         printer.pretty(v)
                                         # We add a comma unconditionally because
-                                        # generated arguments will always be
-                                        # kwargs, so there will always be more
-                                        # to come.
+                                        # generated arguments will always be kwargs,
+                                        # so there will always be more to come.
                                         printer.text(",")
                                         printer.breakable()
 
-                                    # We need to make sure to print these in the
-                                    # argument order for Python 2 and older versions
-                                    # of Python 3.5. In modern versions this isn't
-                                    # an issue because kwargs is ordered.
-                                    arg_order = {
-                                        v: i
-                                        for i, v in enumerate(
-                                            getfullargspec(self.test).args
-                                        )
-                                    }
-                                    for i, (k, v) in enumerate(
-                                        sorted(
-                                            kwargs.items(),
-                                            key=lambda t: (
-                                                arg_order.get(t[0], float("inf")),
-                                                t[0],
-                                            ),
-                                        )
-                                    ):
+                                    for i, (k, v) in enumerate(kwargs.items()):
                                         printer.text(k)
                                         printer.text("=")
                                         printer.pretty(v)
@@ -671,15 +661,31 @@ class StateForActualGivenExecution:
         ``StopTest`` must be a fatal error, and should stop the entire engine.
         """
         try:
-            result = self.execute_once(data)
+            trace = frozenset()
+            if (
+                self.failed_normally
+                and Phase.explain in self.settings.phases
+                and sys.gettrace() is None
+                and not PYPY
+            ):  # pragma: no cover
+                # This is in fact covered by our *non-coverage* tests, but due to the
+                # settrace() contention *not* by our coverage tests.  Ah well.
+                tracer = Tracer()
+                try:
+                    sys.settrace(tracer.trace)
+                    result = self.execute_once(data)
+                    if data.status == Status.VALID:
+                        self.explain_traces[None].add(frozenset(tracer.branches))
+                finally:
+                    sys.settrace(None)
+                    trace = frozenset(tracer.branches)
+            else:
+                result = self.execute_once(data)
             if result is not None:
                 fail_health_check(
                     self.settings,
-                    (
-                        "Tests run under @given should return None, but "
-                        "%s returned %r instead."
-                    )
-                    % (self.test.__name__, result),
+                    "Tests run under @given should return None, but "
+                    f"{self.test.__name__} returned {result!r} instead.",
                     HealthCheck.return_value,
                 )
         except UnsatisfiedAssumption:
@@ -720,10 +726,13 @@ class StateForActualGivenExecution:
                 info.__expected_exception = e
                 verbose_report(info.__expected_traceback)
 
-                origin = traceback.extract_tb(tb)[-1]
-                filename = origin[0]
-                lineno = origin[1]
-                data.mark_interesting((type(e), filename, lineno))
+                self.failed_normally = True
+
+                interesting_origin = get_interesting_origin(e)
+                if trace:  # pragma: no cover
+                    # Trace collection is explicitly disabled under coverage.
+                    self.explain_traces[interesting_origin].add(trace)
+                data.mark_interesting(interesting_origin)
 
     def run_engine(self):
         """Run the test function many times, on database input and generated
@@ -774,8 +783,6 @@ class StateForActualGivenExecution:
         # The engine found one or more failures, so we need to reproduce and
         # report them.
 
-        self.failed_normally = True
-
         flaky = 0
 
         if runner.best_observed_targets:
@@ -783,6 +790,7 @@ class StateForActualGivenExecution:
                 report(line)
             report("")
 
+        explanations = explanatory_lines(self.explain_traces, self.settings)
         for falsifying_example in self.falsifying_examples:
             info = falsifying_example.extra_information
 
@@ -806,6 +814,10 @@ class StateForActualGivenExecution:
                     "assumptions on the first run now fails it."
                 )
             except BaseException as e:
+                # If we have anything for explain-mode, this is the time to report.
+                for line in explanations[falsifying_example.interesting_origin]:
+                    report(line)
+
                 if len(self.falsifying_examples) <= 1:
                     # There is only one failure, so we can report it by raising
                     # it directly.
@@ -849,16 +861,12 @@ class StateForActualGivenExecution:
 
         if flaky > 0:
             raise Flaky(
-                (
-                    "Hypothesis found %d distinct failures, but %d of them "
-                    "exhibited some sort of flaky behaviour."
-                )
-                % (len(self.falsifying_examples), flaky)
+                f"Hypothesis found {len(self.falsifying_examples)} distinct failures, "
+                f"but {flaky} of them exhibited some sort of flaky behaviour."
             )
         else:
             raise MultipleFailures(
-                ("Hypothesis found %d distinct failures.")
-                % (len(self.falsifying_examples))
+                f"Hypothesis found {len(self.falsifying_examples)} distinct failures."
             )
 
     def __flaky(self, message):
@@ -1102,8 +1110,9 @@ def given(
                             type(err), err, err.__traceback__
                         )
                         report("".join(tb_lines))
-                    msg = "Hypothesis found %d failures in explicit examples."
-                    raise MultipleFailures(msg % (len(errors)))
+                    raise MultipleFailures(
+                        f"Hypothesis found {len(errors)} failures in explicit examples."
+                    )
                 elif errors:
                     fragments, the_error_hypothesis_found = errors[0]
                     for f in fragments:
@@ -1139,14 +1148,14 @@ def given(
                     if not (state.failed_normally or generated_seed is None):
                         if running_under_pytest:
                             report(
-                                "You can add @seed(%(seed)d) to this test or "
-                                "run pytest with --hypothesis-seed=%(seed)d "
-                                "to reproduce this failure." % {"seed": generated_seed}
+                                f"You can add @seed({generated_seed}) to this test or "
+                                f"run pytest with --hypothesis-seed={generated_seed} "
+                                "to reproduce this failure."
                             )
                         else:
                             report(
-                                "You can add @seed(%d) to this test to "
-                                "reproduce this failure." % (generated_seed,)
+                                f"You can add @seed({generated_seed}) to this test to "
+                                "reproduce this failure."
                             )
                     # The dance here is to avoid showing users long tracebacks
                     # full of Hypothesis internals they don't care about.
@@ -1245,7 +1254,6 @@ def given(
     return run_test_as_given
 
 
-@deprecated_posargs
 def find(
     specifier: SearchStrategy[Ex],
     condition: Callable[[Any], bool],
@@ -1272,7 +1280,7 @@ def find(
         )
     specifier.validate()
 
-    last = []  # type: List[Ex]
+    last: List[Ex] = []
 
     @settings
     @given(specifier)
@@ -1284,8 +1292,11 @@ def find(
     if random is not None:
         test = seed(random.getrandbits(64))(test)
 
-    test._hypothesis_internal_is_find = True
-    test._hypothesis_internal_database_key = database_key
+    # Aliasing as Any avoids mypy errors (attr-defined) when accessing and
+    # setting custom attributes on the decorated function or class.
+    _test: Any = test
+    _test._hypothesis_internal_is_find = True
+    _test._hypothesis_internal_database_key = database_key
 
     try:
         test()
